ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Computes Curves of Growth for R^1/4 and Exponential profiles with given
c flattenings and determines the circularized half-luminosity ratio
c of Bulge+Disk models with given B/D ratios
c Author: R.P. Saglia, saglia@mpe.mpg.de, December 2010
c See Appendix of  Saglia et al. 2010, A&A, 524, A6    
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      Program CogMod
c
c Input parameters:
c aeb: bulge major axis effective scale lenght
c ahd: disk major axis exponential scale lenght
c bab: ellipticity b/a of the bulge
c bad: ellipticity b/a of the disk
c bt:  bulge to total ratio
c  
      implicit double precision (a-h,o-z)
      parameter (pi=3.141592654d0,twopi=2.0d0*pi)
      parameter (npoints=10000)
      external rombint,fb0,fd0
      real*8 aeb,ahd,bab,bad,bt
      real*8 vaeb(10000),vbab(10000),vahd(10000),vbad(10000),vbt(10000)
      integer*4 ind(10000)
      character*80 namlist
c
c Output parameters
c re:  circularized half-luminosity radius
c
      real*8 re,rebulge,redisk
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
c
      common/cogs/rad,flb,fld,flt
      common/ellb/rb,bab
      common/elld/rd,bad
c
      iman=0
      write(*,*)'******************************************************'
      write(*,*)'*          PROGRAM COGMODALL                         *'
      write(*,*)'******************************************************'
      write(*,*)'*       Author: R.P. Saglia, December 2010           *'
      write(*,*)'------------------------------------------------------'
      write(*,*)'See Appendix of  Saglia et al. 2010, A&A, 524, A6     '
      write(*,*)'Computes circularly averaged half-luminosity radii Re ' 
      write(*,*)'of bulge plus disk models, where the bulge is an      '
      write(*,*)'R^1/4 law and the disk as an exponential law.         '
      write(*,*)'The input parameters are:                             '
      write(*,*)'ae: major axis half-luminosity radius of the bulge aeb'
      write(*,*)'be/ae: minor to major axis ratio of the bulge bab     '
      write(*,*)'B/T: bulge to total luminosity ratio bt               '
      write(*,*)'ah: major axis scale-lenght of the disk ahd           '
      write(*,*)'bh/ah: apparent axis ratio of the disk                '
      write(*,*)'                                                      '  
      write(*,*)'The parameters have to be given in the order:         '
      write(*,*)'aeb,ahd,bab,bad,bt                                    '
      write(*,*)'                                                      '  
      write(*,*)'If the list option is chosen, no starting comments are'
      write(*,*)'allowed.                                              '
      write(*,*)'                                                      '
      write(*,*)'The output is:                                        '
      write(*,*)'aeb,bab,rebulge,ahd,bad,redisk,bt,re,Index            '
      write(*,*)'                                                      '
      write(*,*)'where rebulge is the circularized radius of the bulge,'
      write(*,*)'      redisk  is the circularized radius of the disk, '
      write(*,*)'      re  is the circularized radius of the disk of   '
      write(*,*)'      sum of the two components and Index=0 flags     '
      write(*,*)'      failed circularitions and/or bad input values.  '
      write(*,*)'                                                      '
      write(*,*)'The output is written in the file                     '
      write(*,*)'                                                      '
      write(*,*)'manual.ReCirAll                                       '
      write(*,*)'                                                      '
      write(*,*)'if the manual entry is chosen, or                     '
      write(*,*)'                                                      '
      write(*,*)'filename.ReCirAll                                     '
      write(*,*)'                                                      '
      write(*,*)'if a list with name filename has been given.          '
      write(*,*)'******************************************************'
      write(*,*)'                                                      '  
      write(*,*)'Manual entry (0) or list (1)?                         '
      read(*,*),iman
      if(iman.eq.0)then
      write(*,*)'Input aeb,ahd,bab,bad,bt'
      read(*,*)aeb,ahd,bab,bad,bt
      nre=1
      vaeb(1)=aeb
      vahd(1)=ahd
      vbab(1)=bab
      vbad(1)=bad
      vbt(1)=bt
      it=1
      if(vaeb(it).ge.0.and.vahd(it).ge.0.and.vbab(it).ge.0.and.
     1     vbab(it).le.1.and.vbad(it).ge.0.and.vbad(it).le.1.and.
     2     vbt(it).ge.0.and.vbt(it).le.1)then
         ind(it)=1
      else
         ind(1)=0
      endif
c
c Compute cogs for bulge and disk
c 
c      step=0.0008d0*ahd/aeb
      step=0.1
      do i=1,npoints
         rad(i)=step*i
         flb(i)=fluxb(rad(i))/twopi
         fld(i)=fluxd(rad(i))/twopi
      enddo
c      write(*,*)'Rmax=',rad(npoints)
c
      open(unit=1,file='bulge.cogmod',status='unknown')
      open(unit=2,file='disk.cogmod',status='unknown')
      write(1,*)'Bulge ae=',aeb,' b/a=',bab
      write(2,*)'Disk ah=',ahd,' b/a=',bad
      do i=1,npoints
         write(1,*)aeb*rad(i),flb(i),fluxb0(rad(i))
         write(2,*)ahd*rad(i),fld(i),fluxd0(rad(i))
      enddo
      close(unit=1)
      close(unit=2)
      else
         write(*,*)'List name?'
         read(*,'(a)')namlist
         call findspace(namlist,nspace)
         open(unit=10,file=namlist,status='old')
         it=1
         do i=1,10000
            read(10,*,end=11)vaeb(it),vahd(it),vbab(it),vbad(it),vbt(it)
            if(vaeb(it).ge.0.and.vahd(it).ge.0.and.vbab(it).ge.0.and.
     1         vbab(it).le.1.and.vbad(it).ge.0.and.vbad(it).le.1.and.
     2         vbt(it).ge.0.and.vbt(it).le.1)then
               ind(it)=1
            else
               ind(it)=0
            endif
            it=it+1
         enddo
 11      close(unit=10)
         nre=it-1
         if(nre.gt.10000)then
            write(*,*)nre,' graeter than 10000, set to 10000'
            nre=10000
         else
            write(*,*)nre,' galaxies to compute'
         endif
c         step=0.0008d0*ahd/aeb
         step=0.1
         do i=1,npoints
            rad(i)=step*i
         enddo
      endif
c
c Compute re 
c
      if(iman.eq.1)then
       open(unit=1,file=namlist(1:nspace)//'.ReCirAll',status='unknown')
      else
       open(unit=1,file='manual.ReCirAll',status='unknown')
      endif
      write(1,*)'    ae  (b/a)B ReCirB   ah  (b/a)D ReCirD  B/T  ReCir'
     1//' Index'
      ifirst=1
      isecond=1
      ithird=1
      do j=1,nre
         frac=1.d0*j/nre
c         write(*,*)'i=',j,' out of ',nre,' fraction=',frac
         if(frac.gt..25.and.frac.lt..5.and.ifirst.eq.1)then
            write(*,*)'Done 25%'
            ifirst=0
         endif
         if(frac.gt..50.and.frac.lt..75.and.isecond.eq.1)then
            write(*,*)'Done 50%'
            isecond=0
         endif
         if(frac.gt..75.and.frac.lt.1.and.ithird.eq.1)then
            write(*,*)'Done 75%'
            ithird=0
         endif
         aeb=vaeb(j)
         ahd=vahd(j)
         bt=vbt(j)
         bab=vbab(j)
         bad=vbad(j)
         if(ind(j).eq.1)then
         do i=1,npoints
            flb(i)=fluxb(rad(i))/twopi
            fld(i)=fluxd(rad(i))/twopi
         enddo
c
         if(aeb.gt.0)then
           call recir(aeb,ahd,1.d0,rebulge)
         else
           rebulge=0.d0
          endif
          if(ahd.gt.0)then
           call recir(aeb,ahd,0.d0,redisk)
          else
            redisk=0.d0
          endif
          if(aeb.gt.0.and.ahd.gt.0)then
            call recir(aeb,ahd,bt,re)
          else
             if(aeb.eq.0.d0)re=redisk
             if(ahd.eq.0.d0)re=rebulge
          endif
c
         else
            rebulge=-99.9
            redisk=-99.9
            re=-99.9
         endif
c         if(iman.eq.0)then
      write(*,*)'******************************************************'
      write(*,*)'* Bulge MJ ae=',aeb,' b/a=',bab,' Re Bulge=',rebulge
      write(*,*)'* Disk  MJ h =',ahd,' b/a=',bad,' Re Disk=',redisk
      write(*,*)'* Bulge/Total=',bt,' Re total=',re
      write(*,*)'******************************************************'
c         endif
         write(1,101)aeb,bab,rebulge,ahd,bad,redisk,bt,re,ind(j)
 101     format(f7.2,f6.3,f7.2,f7.2,f6.2,f7.2,f6.3,f7.2,i6)
      if(iman.eq.0)then
      open(unit=11,file='bulgedisk.cogmod',status='unknown')
      write(11,*)'Bulge ae=',aeb,' b/a=',bab,' Disk ah=',ahd,' b/a=',
     1bad,' B/T=',bt
      do i=1,npoints
         write(11,*)aeb*rad(i),flt(i)
      enddo
      close(unit=11)
      endif
      enddo
      close(unit=1)
      stop
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Finds first space character in a string
c
      subroutine findspace(nam,nnam)
      character*80 nam
      do i=1,80
         if(nam(i:i).eq.' ')goto 1
      enddo
 1    nnam=i-1
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Computes half-luminosity radius of the given cog
c
      subroutine recir(aeb,ahd,bt,retot)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=10000)
      real*8 aeb,ahd,bt,retot
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
c
      common/cogs/rad,flb,fld,flt
      common/ellb/rb,bab
      common/elld/rd,bad
c
      if(bt.eq.1.)then
         retot=aeb*findhalf(flb)
         return
      endif
c
      if(bt.eq.0.)then
         retot=ahd*findhalf(fld)
         return
      endif
c
c      write(*,*)'calling fldisk'
      call fldisk(aeb,ahd,bt)
c      write(*,*)'calling findhalf'
      retot=aeb*findhalf(flt)
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function findhalf(fl)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=10000)
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
      real*8 fl(npoints)
c      
      common/cogs/rad,flb,fld,flt
c
      if(fl(npoints).lt..5)then
         findhalf=-1.d0
         goto 1
      endif
      do i=1,npoints
         if(fl(i).gt.0.5)then
            findhalf=rad(i-1)+(rad(i)-rad(i-1))
     1               *(0.5-fl(i-1))/(fl(i)-fl(i-1))
            goto 1
         endif
      enddo
c
 1    return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine fldisk(aeb,ahd,bt)
c
      implicit double precision (a-h,o-z)
      parameter (pi=3.141592654d0,twopi=2.0d0*pi)
      parameter (npoints=10000)
      real*8 aeb,ahd,bt,re
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
c
      common/cogs/rad,flb,fld,flt
c
c      open(unit=99,file='junk.dat',status='unknown')
c      write(99,*)'ahd=',ahd
      do i=1,npoints
         fldsk=fluxd(rad(i)*aeb/ahd)/twopi
         flt(i)=bt*flb(i)+(1.d0-bt)*fldsk
c         write(99,*)rad(i),flb(i),fldsk,flt(i)
c         write(*,*)rad(i),flb(i),fldsk,flt(i)
      enddo
c
c      close(unit=99)
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fluxb(x)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=10000)
      real*8 rb,bab
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
      real*8 pi,tol
c
      external rombint,fb0
c
      common/cogs/rad,flb,fld,flt
      common/ellb/rb,bab
c
c
      rb=x
      pi=3.141592654d0
      tol=1.d-5
      fluxb=4.d0*rombint(fb0,0.d0,pi/2.d0,tol)*bab
c
c      if(fluxb.gt.1.d0)write(*,*)'x=',x,' fluxb=',fluxb
c
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fluxd(x)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=10000)
      real*8 rd,bad
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
      real*8 pi,tol
c
      external rombint,fd0
c
      common/cogs/rad,flb,fld,flt
      common/elld/rd,bad
c
      rd=x
      pi=3.141592654d0
      tol=1.d-5
c
      fluxd=4.d0*rombint(fd0,0.d0,pi/2.d0,tol)*bad
c
c      if(fluxd.gt.1.d0)write(*,*)'x=',x,' fluxd=',fluxd
c
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fb0(phi)
c
      implicit double precision (a-h,o-z)
      real*8 rb,rr,bab,phi,geo
c
      common/ellb/rb,bab
c
      geo=1.-(1.-bab**2)*cos(phi)**2
      rr=rb/bab*geo**.5
      fb0=fluxb0(rr)/geo
c
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fd0(phi)
c
      implicit double precision (a-h,o-z)
      real*8 rd,rr,bad,phi,geo
c
      common/elld/rd,bad
c
      geo=1.-(1.-bad**2)*cos(phi)**2
      rr=rd/bad*geo**.5
      fd0=fluxd0(rr)/geo
c
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  Functions sbd0, fluxd0, sbd, and fluxd, for exponential disk.
c  Initialize with call sbdini(gamma).
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        function sbd0(r)
c  Compute the surface brightness distribution for an exponential disk.
c  r is in units of the disk scale length a.  Total flux is normalized to 1.
c
        implicit double precision (a-h,o-z)
        parameter (pi=3.141592654d0,twopi=2.0d0*pi)
c
        sbd0=exp(-r)/twopi
        return
        end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        function fluxd0(r)
c  Compute the integrated flux distribution for an exponential disk.
c  r is in units of the disk scale length a.  Total flux is normalized to 1.
c
        implicit double precision (a-h,o-z)
c
        fluxd0=1.0-(1.0+r)*exp(-r)
        return
        end
c  Functions sbb0, fluxb0, sbb, and fluxb, for r^{1/4}-law.
c  Initialize with call sbbini(gamma).
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        function sbb0(r)
c  Compute the surface brightness distribution for the r**(1/4) law.
c  r is in units of the half-light radius Re.  Total flux is normalized to 1.
c
        implicit double precision (a-h,o-z)
        parameter (pi8=8.0d0*3.141592654d0,r0=3459.4850148d0)
c
        x=sqrt(sqrt(r*r0))
        sbb0=r0*r0*exp(-x)/(5040.0d0*pi8)
        return
        end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        function fluxb0(r)
c  Computes the integrated flux distribution for the r**(1/4) law.
c  r is in units of the half-light radius Re.  Total flux is normalized to 1.
c
        implicit double precision (a-h,o-z)
c
        z=7.67d0*sqrt(sqrt(r))
          fluxb0=1.d0-exp(-z)*(1.d0+z+z*z/2.d0+z*z*z/6.d0+
     1           z*z*z*z/24.d0+z*z*z*z*z/120.d0+
     2           z*z*z*z*z*z/720.d0+z*z*z*z*z*z*z/5040.d0)
        return
        end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

	function rombint(f,a,b,tol)
c  Rombint returns the integral from a to b of f(x)dx using Romberg integration.
c  The method converges provided that f(x) is continuous in (a,b).  The function
c  f must be double precision and must be declared external in the calling
c  routine.  tol indicates the desired relative accuracy in the integral.
c
	parameter (MAXITER=17,MAXJ=5)
	implicit double precision (a-h,o-z)
	dimension g(MAXJ+1)
	external f
c
	h=0.5d0*(b-a)
	gmax=h*(f(a)+f(b))
	g(1)=gmax
	nint=1
	error=1.0d20
	i=0
10	  i=i+1
	  if (i.gt.MAXITER.or.(i.gt.5.and.abs(error).lt.tol))
     2      go to 40
c  Calculate next trapezoidal rule approximation to integral.
	  g0=0.0d0
	    do 20 k=1,nint
	    g0=g0+f(a+(k+k-1)*h)
20	  continue
	  g0=0.5d0*g(1)+h*g0
	  h=0.5d0*h
	  nint=nint+nint
	  jmax=min(i,MAXJ)
	  fourj=1.0d0
	    do 30 j=1,jmax
c  Use Richardson extrapolation.
	    fourj=4.0d0*fourj
	    g1=g0+(g0-g(j))/(fourj-1.0d0)
	    g(j)=g0
	    g0=g1
30	  continue
	  if (abs(g0).gt.tol) then
	    error=(g0-gmax)/g0
	  else
	    error=(g0-gmax)/tol
	  end if
	  gmax=g0
	  g(jmax+1)=g0
	go to 10
40	rombint=g0
	if (i.gt.MAXITER.and.abs(error).gt.tol)
     2    write(*,*) 'Rombint failed to converge; integral, error=',
     3    rombint,error
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

	subroutine splder(y,dy,n)
c  Splder fits a cubic spline to y and returns the first derivatives at
c  the grid points in dy.  Dy is equivalent to a 4th-order Pade
c  difference formula for dy/di.
c
	implicit double precision (a-h,o-z)
	dimension f(3001),g(3001),y(n),dy(n)
	common /spline/ g
c
	n1=n-1
	if (n1.gt.3000) write(*,*) 'Spline array overflow!!! n1=',n1,'>3000'
c  Quartic fit to dy/di at boundaries, assuming d3y/di3=0.
	f(1)=(-10.0d0*y(1)+15.0d0*y(2)-6.0d0*y(3)+y(4))/6.0d0
	f(n)=(10.0d0*y(n)-15.0d0*y(n1)+6.0d0*y(n-2)-y(n-3))/6.0d0
c  Solve the tridiagonal system
c  dy(i-1)+4*dy(i)+dy(i+1)=3*(y(i+1)-y(i-1)), i=2,3,...,n1,
c  with dy(1)=f(1), dy(n)=f(n).
	  do 10 i=2,n1
	  f(i)=g(i)*(3.0d0*(y(i+1)-y(i-1))-f(i-1))
10	continue
	dy(n)=f(n)
	  do 20 i=n1,1,-1
	  dy(i)=f(i)-g(i)*dy(i+1)
20	continue
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	subroutine splini
c  Splini must be called before splder to initialize array g in common.
c
	implicit double precision (a-h,o-z)
	dimension g(3001)
	common /spline/ g
	save /spline/
c
	g(1)=0.0d0
	  do 10 i=2,3001
	  g(i)=1.0d0/(4.0d0-g(i-1))
10	continue
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 
	function dozero(re,rexp,csbexp)
c
	implicit double precision (a-h,o-z)
	dimension value(100)
	fluxhalf=(re*re+csbexp*rexp*rexp)/2.d0
	if(re.gt.rexp*1.677d0.and.re.lt.rexp*1.678d0)then
	   dozero=re
	   return
	endif
	if(re.gt.rexp*1.679d0)then
	   start0=rexp*1.679
	   end0=re
	endif
	if(start0.lt.rexp*1.677d0)then
	   start0=re
	   end0=rexp*1.677d0
	endif
	checkstart=re*re*fluxb0(start0/re)
     1             +rexp*rexp*csbexp*fluxd0(start0/rexp)
     2             -fluxhalf
	if(checkstart.gt.0)then
	   write(*,*)'start>0!'
	   dozero=0.d0
	   return
	endif
	checkend=re*re*fluxb0(end0/re)
     1             +rexp*rexp*csbexp*fluxd0(end0/rexp)
     2             -fluxhalf
	if(checkend.lt.0)then
	   write(*,*)'end<0!'
	   dozero=0.d0
	   return
	endif
c	write(*,*)'check start,end',checkstart,checkend
	step=(end0-start0)/100.d0
	pos=start0
	do 1 i=1,101
	 pos=pos+step
	 value(i)=re*re*fluxb0(pos/re)
     1         +rexp*rexp*csbexp*fluxd0(pos/rexp)
     2             -fluxhalf   
	 if(value(i).gt.0.d0)then
	    dozero=pos-step*value(i)/(value(i)-value(i-1))
	    return
	 endif
 1	continue
	write(*,*)'Value always negative!'
	dozero=0.
	return
	end
