ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Computes Curves of Growth for two Sersic Profiles with given
c flattenings and determines the circularized half-luminosity ratio
c of Bulge+Disk models with given B/D ratios
c Author: R.P. Saglia, saglia@mpe.mpg.de, December 2010
c See Appendix of  Saglia et al. 2010, A&A, 524, A6    
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      Program CogModAllSersic
c
c Input parameters:
c aeb: bulge major axis effective scale lenght
c ahd: disk major axis exponential scale lenght
c bab: ellipticity b/a of the bulge
c bad: ellipticity b/a of the disk
c nSerB: nSersic of Bulge
c nSerD: nSersic of Disk
c bt:  bulge to total ratio
c  
      implicit double precision (a-h,o-z)
      parameter (pi=3.141592654d0,twopi=2.0d0*pi)
      parameter (npoints=10000)
      external rombint,fb0,fd0
      real*8 aeb,ahd,bab,bad,bt,nSerB,nSerD
      real*8 vaeb(10000),vbab(10000),vahd(10000),vbad(10000),vbt(10000)
      real*8 vnSerB(10000),vnSerD(10000)
      integer*4 ind(10000)
      character*80 namlist
c
c Output parameters
c re:  circularized half-luminosity radius
c
      real*8 re,rebulge,redisk
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
c
      common/cogs/rad,flb,fld,flt
      common/ellb/rb,bab,nSerB
      common/elld/rd,bad,nSerD
      common/nserB/alphaB,rkappaB
      common/nserD/alphaD,rkappaD
c
      write(*,*)'******************************************************'
      write(*,*)'*          PROGRAM COGMODALLSERSIC                   *'
      write(*,*)'******************************************************'
      write(*,*)'*       Author: R.P. Saglia, December 2010           *'
      write(*,*)'------------------------------------------------------'
      write(*,*)'See Appendix of  Saglia et al. 2010, A&A, 524, A6     '
      write(*,*)'Computes circularly averaged half-luminosity radii Re ' 
      write(*,*)'of bulge plus disk models, where the bulge and the    '
      write(*,*)'disk are two Sersic profiles.                         '
      write(*,*)'The input parameters are:                             '
      write(*,*)'ae: major axis half-luminosity radius of the bulge aeb'
      write(*,*)'be/ae: minor to major axis ratio of the bulge bab     '
      write(*,*)'nSerB: Sersic index of the Bulge                      '
      write(*,*)'B/T: bulge to total luminosity ratio bt               '
      write(*,*)'ah: major axis half-luminosity radius of the disk ahd '
      write(*,*)'bh/ah: apparent axis ratio of the disk                '
      write(*,*)'nSerD: Sersic index of the Disk                       '
      write(*,*)'                                                      '  
      write(*,*)'The parameters have to be given in the order:         '
      write(*,*)'aeb,ahd,bab,bad,SerB,SerD,bt                          '
      write(*,*)'                                                      '  
      write(*,*)'If the list option is chosen, no starting comments are'
      write(*,*)'allowed.                                              '
      write(*,*)'                                                      '
      write(*,*)'The output is:                                        '
      write(*,*)'aeb,bab,nSerB,rebulge,ahd,bad,nSerD,redisk,bt,re,Index'
      write(*,*)'                                                      '
      write(*,*)'where rebulge is the circularized radius of the bulge,'
      write(*,*)'      redisk  is the circularized radius of the disk, '
      write(*,*)'      re  is the circularized radius of the disk of   '
      write(*,*)'      sum of the two components and Index=0 flags     '
      write(*,*)'      failed circularitions and/or bad input values.  '
      write(*,*)'                                                      '
      write(*,*)'The output is written in the file                     '
      write(*,*)'                                                      '
      write(*,*)'manual.ReCirAllSersic                                 '
      write(*,*)'                                                      '
      write(*,*)'if the manual entry is chosen, or                     '
      write(*,*)'                                                      '
      write(*,*)'filename.ReCirAllSersic                               '
      write(*,*)'                                                      '
      write(*,*)'if a list with name filename has been given.          '
      write(*,*)'                                                      '
      write(*,*)'Two equally good options are given to compute ReCir:  '
      write(*,*)'Grid:   ReCir is computed by interpolation of a cog   ' 
      write(*,*)'        computed on a radial grid with steps 1/10     '
      write(*,*)'        of the scale lenghts of the components        '
      write(*,*)'Search: ReCir is computed with the Newtown method     '
      write(*,*)'                                                      '
      write(*,*)'******************************************************'
      write(*,*)'                                                      '  
      iman=0
      write(*,*)'Manual entry (0) or list (1)? [0]'
      read(*,*)iman
      imet=0
      write(*,*)'Method to find Retot (0=Grid, 1=Search)? [0]'
      read(*,*)imet
      if(iman.eq.0)then
      write(*,*)'Input aeb,ahd,bab,bad,nSerB,nSerD,bt'
      read(*,*)aeb,ahd,bab,bad,nSerB,nSerD,bt
      nre=1
      vaeb(1)=aeb
      vahd(1)=ahd
      vbab(1)=bab
      vbad(1)=bad
      vbt(1)=bt
      vnSerB(1)=nSerB
      vnSerD(1)=nSerD
      it=1
      if(vaeb(it).ge.0.and.vahd(it).ge.0.and.vbab(it).ge.0.and.
     1     vbab(it).le.1.and.vbad(it).ge.0.and.vbad(it).le.1.and.
     2     vbt(it).ge.0.and.vbt(it).le.1)then
         ind(it)=1
      else
         ind(1)=0
      endif
c
c Compute cogs for bulge and disk
c 
      step=0.01
      nSerB=vnSerB(it)
      alphaB=nSerB
      rkappaB=1.9992d0*alphaB-0.3271d0
      nSerD=vnSerD(it)
      alphaD=nSerD
      rkappaD=1.9992d0*alphaD-0.3271d0
c      write(99,*)'alphaB=',alphaB,' kappaB=',rkappaB,' alphaD=',alphaD,
c     1           ' kappaD=',rkappaD
      do i=1,npoints
         rad(i)=step*i
         flb(i)=fluxb(rad(i))/twopi
         fld(i)=fluxd(rad(i))/twopi
c         write(99,*)rad(i),flb(i),fld(i)
      enddo
c      write(*,*)'Rmax=',rad(npoints)
      open(unit=1,file='bulge.cogmod',status='unknown')
      open(unit=2,file='disk.cogmod',status='unknown')
      write(1,*)'Bulge ae=',aeb,' b/a=',bab,' nSer=',nSerB
      write(2,*)'Disk ae=',ahd,' b/a=',bad,' nSer=',nSerD
      do i=1,npoints
         write(1,*)aeb*rad(i),flb(i),fluxb0(rad(i))
         write(2,*)ahd*rad(i),fld(i),fluxd0(rad(i))
      enddo
      close(unit=1)
      close(unit=2)
      else
         write(*,*)'List name?'
         read(*,'(a)')namlist
         call findspace(namlist,nspace)
         open(unit=10,file=namlist,status='old')
         it=1
         do i=1,10000
            read(10,*,end=11)vaeb(it),vahd(it),vbab(it),vbad(it),
     1                       vnSerB(it),vnSerD(it),vbt(it)
            if(vaeb(it).ge.0.and.vahd(it).ge.0.and.vbab(it).ge.0.and.
     1         vbab(it).le.1.and.vbad(it).ge.0.and.vbad(it).le.1.and.
     2         vbt(it).ge.0.and.vbt(it).le.1)then
               ind(it)=1
            else
               ind(it)=0
            endif
            it=it+1
         enddo
 11      close(unit=10)
         nre=it-1
         if(nre.gt.10000)then
            write(*,*)nre,' graeter than 10000, set to 10000'
            nre=10000
         else
            write(*,*)nre,' galaxies to compute'
         endif
c         step=0.0008d0*ahd/aeb
         step=0.1
         do i=1,npoints
            rad(i)=step*i
         enddo
      endif
c
c Compute re 
c
      if(iman.eq.1)then
       open(unit=1,file=namlist(1:nspace)//'.ReCirAllSersic',
     1      status='unknown')
      else
       open(unit=1,file='manual.ReCirAllSersic',status='unknown')
      endif
      if(imet==0)then
      write(1,*)'    ae  (b/a)B nSerB ReCirB   ah  (b/a)D nSerD ReCirD'
     1//'  B/T  ReCir Index Grid'
      else
      write(1,*)'    ae  (b/a)B nSerB ReCirB   ah  (b/a)D nSerD ReCirD'
     1//'  B/T  ReCir Index Search'
      endif
      ifirst=1
      isecond=1
      ithird=1
      do j=1,nre
         frac=1.d0*j/nre
c         write(*,*)'i=',j,' out of ',nre,' fraction=',frac
         if(frac.gt..25.and.frac.lt..5.and.ifirst.eq.1)then
            write(*,*)'Done 25%'
            ifirst=0
         endif
         if(frac.gt..50.and.frac.lt..75.and.isecond.eq.1)then
            write(*,*)'Done 50%'
            isecond=0
         endif
         if(frac.gt..75.and.frac.lt.1.and.ithird.eq.1)then
            write(*,*)'Done 75%'
            ithird=0
         endif
         aeb=vaeb(j)
         ahd=vahd(j)
         bt=vbt(j)
         bab=vbab(j)
         bad=vbad(j)
         nSerB=vnSerB(j)
         alphaB=nSerB
         rkappaB=1.9992d0*alphaB-0.3271d0
         nSerD=vnSerD(j)
         alphaD=nSerD
         rkappaD=1.9992d0*alphaD-0.3271d0
         if(ind(j).eq.1)then
         do i=1,npoints
            flb(i)=fluxb(rad(i))/twopi
            fld(i)=fluxd(rad(i))/twopi
         enddo
c
         if(aeb.gt.0)then
           call recir(aeb,ahd,1.d0,rebulge)
         else
           rebulge=0.d0
          endif
          if(ahd.gt.0)then
           call recir(aeb,ahd,0.d0,redisk)
          else
            redisk=0.d0
          endif
          if(aeb.gt.0.and.ahd.gt.0)then
            if(imet.eq.0)call recir(aeb,ahd,bt,re)
            if(imet.eq.1)call recirsearch(aeb,ahd,bt,rebulge,redisk,re,
     1                                    tol,iter)
          else
             if(aeb.eq.0.d0)re=redisk
             if(ahd.eq.0.d0)re=rebulge
          endif
c
         else
            rebulge=-99.9
            redisk=-99.9
            re=-99.9
         endif
c         if(iman.eq.0)then
      write(*,*)'******************************************************'
      write(*,*)'* Bulge MJ ae=',aeb,' b/a=',bab,' nSer Bulge=',nSerB,
     1' Re Bulge=',rebulge
      write(*,*)'* Disk  MJ ae =',ahd,' b/a=',bad,' nSer Disk=',nSerD, 
     1' Re Disk=',redisk
      write(*,*)'* Bulge/Total=',bt,' Re total=',re
      write(*,*)'******************************************************'
c         endif
         write(1,101)aeb,bab,nSerB,rebulge,ahd,bad,nSerD,redisk,bt,re,
     1               ind(j)
 101     format(f7.2,f6.3,f6.3,f7.2,f7.2,f6.2,f6.3,f7.2,f6.3,f7.2,i6)
      if(iman.eq.0)then
      open(unit=11,file='bulgedisk.cogmod',status='unknown')
      write(11,*)'Bulge ae=',aeb,' b/a=',bab,' nSerB=',nSerB,
     1' Disk ah=',ahd,' b/a=',bad,' nSerD=',nSerD,' B/T=',bt
      do i=1,npoints
         write(11,*)aeb*rad(i),flt(i)
      enddo
      close(unit=11)
      endif
      enddo
      close(unit=1)
      stop
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Finds first space character in a string
c
      subroutine findspace(nam,nnam)
      character*80 nam
      do i=1,80
         if(nam(i:i).eq.' ')goto 1
      enddo
 1    nnam=i-1
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Computes half-luminosity radius of the given cog
c
      subroutine recir(aeb,ahd,bt,retot)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=10000)
      real*8 aeb,ahd,bt,retot
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
      real*8 nSerB,nSerD
c
      common/cogs/rad,flb,fld,flt
      common/ellb/rb,bab,nSerB
      common/elld/rd,bad,nSerD
c
      if(bt.eq.1.)then
         retot=aeb*findhalf(flb)
         return
      endif
c
      if(bt.eq.0.)then
         retot=ahd*findhalf(fld)
         return
      endif
c
c      write(*,*)'calling fldisk'
      call fldisk(aeb,ahd,bt)
c      write(*,*)'calling findhalf'
      retot=aeb*findhalf(flt)
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Computes half-luminosity radius of the given cog
c
      subroutine recirsearch(aeb,ahd,bt,rebulge,redisk,retot,tol,iter)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=10000)
      real*8 aeb,ahd,bt,retot
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
      real*8 nSerB,nSerD
c
      common/cogs/rad,flb,fld,flt
      common/ellb/rb,bab,nSerB
      common/elld/rd,bad,nSerD
c
      if(bt.eq.1.)then
         retot=rebulge
         return
      endif
c
      if(bt.eq.0.)then
         retot=redisk
         return
      endif
c
      tol=1.e-5
      itermax=1000
      iter=1
      retot0=(rebulge+redisk)/2
      relow=redisk
      rehigh=rebulge
      if(redisk.gt.rebulge)then
         relow=rebulge
         rehigh=redisk
      endif
      f1=fluxtot(relow,aeb,ahd,bt)-0.5d0
      f2=fluxtot(rehigh,aeb,ahd,bt)-0.5d0
      write(99,*)iter,relow,f1,rehigh,f2
 1    if(f1.gt.0)then
         write(*,*)'f1>0! Tol=-99'
         write(*,*)'iter=',iter,' relow=',relow,' f1=',f1
         retot=retot0
         tol=-99.
         return
      endif
       if(f2.lt.0)then
         write(*,*)'f2<0! Tol=+99'
         retot=retot0
         tol=99.
         return
      endif
      if(f1.eq.0.or.f2.eq.0)then
         retot=retot0
         tol=0.
         return
      endif
      fmed=fluxtot(retot0,aeb,ahd,bt)-0.5d0
      if(abs(fmed).lt.tol)then
         retot=retot0
         tol=fmed
         return
      endif
      if(fmed.gt.0.)then
         rehigh=retot0
         f2=fmed
      endif
      if(fmed.lt.0.)then
         relow=retot0
         f1=fmed
      endif
      retot0=(relow+rehigh)/2.
      iter=iter+1
      write(99,*)iter,relow,f1,rehigh,f2
      if(iter.gt.itermax)then
         retot=retot0
         tol=fmed+itermax
         write(*,*)'Iter>1000, tol=',fmed
         return
      endif
      goto 1
c
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function findhalf(fl)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=10000)
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
      real*8 fl(npoints)
c      
      common/cogs/rad,flb,fld,flt
c
      if(fl(npoints).lt..5)then
         findhalf=-1.d0
         goto 1
      endif
      do i=1,npoints
         if(fl(i).gt.0.5)then
            findhalf=rad(i-1)+(rad(i)-rad(i-1))
     1               *(0.5-fl(i-1))/(fl(i)-fl(i-1))
            goto 1
         endif
      enddo
c
 1    return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine fldisk(aeb,ahd,bt)
c
      implicit double precision (a-h,o-z)
      parameter (pi=3.141592654d0,twopi=2.0d0*pi)
      parameter (npoints=10000)
      real*8 aeb,ahd,bt,re
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
c
      common/cogs/rad,flb,fld,flt
c
c      open(unit=99,file='junk.dat',status='unknown')
c      write(99,*)'ahd=',ahd
      do i=1,npoints
         fldsk=fluxd(rad(i)*aeb/ahd)/twopi
         flt(i)=bt*flb(i)+(1.d0-bt)*fldsk
c         write(99,*)rad(i),flb(i),fldsk,flt(i)
c         write(*,*)rad(i),flb(i),fldsk,flt(i)
      enddo
c
c      close(unit=99)
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fluxtot(ra,aeb,ahd,bt)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=10000)
      real*8 rb,bab,ra
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
      real*8 pi,tol,nSerB,nSerD
      parameter (pi=3.141592654d0,twopi=2.0d0*pi)
c
      external rombint,fb0
c
      common/cogs/rad,flb,fld,flt
      common/ellb/rb,bab,nSerB
      common/nserB/alphaB,rkappaB
      common/elld/rd,bad,nSerD
      common/nserD/alphaD,rkappaD
c
c
      fB=fluxb(ra/aeb)/twopi
      fD=fluxd(ra/ahd)/twopi
      fluxtot=bt*fB+(1.d0-bt)*fD
      write(99,*)'ra=',ra
      write(99,*)'ra/aeb=',ra/aeb,' aeb=',aeb,' bab=',bab,' fB=',fB
      write(99,*)'ra/ahd=',ra/ahd,' ahd=',ahd,' bad=',
     1           bad,' fD=',fD
      write(99,*)'b/t=',bt,' ftot=',fluxtot
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fluxb(x)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=10000)
      real*8 rb,bab
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
      real*8 pi,tol,nSerB
c
      external rombint,fb0
c
      common/cogs/rad,flb,fld,flt
      common/ellb/rb,bab,nSerB
      common/nserB/alphaB,rkappaB
c
c
      rb=x
      pi=3.141592654d0
      tol=1.d-5
      fluxb=4.d0*rombint(fb0,0.d0,pi/2.d0,tol)*bab
c      write(99,*)'x=',x,' fluxb=',fluxb,' bab=',bab
c
c      if(fluxb.gt.1.d0)write(*,*)'x=',x,' fluxb=',fluxb
c
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fluxd(x)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=10000)
      real*8 rd,bad
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),fld(npoints),flt(npoints)
      real*8 pi,tol,nSerD
c
      external rombint,fd0
c
      common/cogs/rad,flb,fld,flt
      common/elld/rd,bad,nSerD
      common/nserD/alphaD,rkappaD
c
      rd=x
      pi=3.141592654d0
      tol=1.d-5
c
      fluxd=4.d0*rombint(fd0,0.d0,pi/2.d0,tol)*bad
c      write(99,*)'x=',x,' fluxd=',fluxd,' alphaD=',alphaD,' rkappaD=',
c     1           rkappaD
c
c      if(fluxd.gt.1.d0)write(*,*)'x=',x,' fluxd=',fluxd
c
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fb0(phi)
c
      implicit double precision (a-h,o-z)
      real*8 rb,rr,bab,phi,geo,nSerB
c
      common/ellb/rb,bab,nSerB
      common/nserB/alphaB,rkappaB
c
      geo=1.-(1.-bab**2)*cos(phi)**2
      rr=rb/bab*geo**.5
      fb0=fluxb0(rr)/geo
c      write(99,*)'Bulge phi=',phi,' rr=',rr,' geo=',geo,' fb0=',fb0
c
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fd0(phi)
c
      implicit double precision (a-h,o-z)
      real*8 rd,rr,bad,phi,geo,nSerD
c
      common/elld/rd,bad,nSerD
      common/nserD/alphaD,rkappaD
c
      geo=1.-(1.-bad**2)*cos(phi)**2
      rr=rd/bad*geo**.5
      fd0=fluxd0(rr)/geo
c      write(99,*)'Disk phi=',phi,' rr=',rr,' geo=',geo,' fd0=',fd0
c
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        function fluxb0(r)
c  Compute the integrated flux distribution for Sersic profile.
c  r is in units of half-luminosity radius.  Total flux is normalized to 1.
c
        implicit double precision (a-h,o-z)
c
        common/nserB/alphaB,rkappaB
c
        X=rkappaB*r**(1.d0/alphaB)
        A=2.d0*alphaB
        CALL INCOG(A,X,GIN,GIM,GIP)
        fluxb0=GIP
        return
        end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        function fluxd0(r)
c  Compute the integrated flux distribution for Sersic profile.
c  r is in units of half-luminosity radius.  Total flux is normalized to 1.
c
        implicit double precision (a-h,o-z)
c
        common/nserD/alphaD,rkappaD
c
        X=rkappaD*r**(1.d0/alphaD)
        A=2.d0*alphaD
        CALL INCOG(A,X,GIN,GIM,GIP)
        fluxd0=GIP
        return
        end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

	function rombint(f,a,b,tol)
c  Rombint returns the integral from a to b of f(x)dx using Romberg integration.
c  The method converges provided that f(x) is continuous in (a,b).  The function
c  f must be double precision and must be declared external in the calling
c  routine.  tol indicates the desired relative accuracy in the integral.
c
	parameter (MAXITER=17,MAXJ=5)
	implicit double precision (a-h,o-z)
	dimension g(MAXJ+1)
	external f
c
	h=0.5d0*(b-a)
	gmax=h*(f(a)+f(b))
	g(1)=gmax
	nint=1
	error=1.0d20
	i=0
10	  i=i+1
	  if (i.gt.MAXITER.or.(i.gt.5.and.abs(error).lt.tol))
     2      go to 40
c  Calculate next trapezoidal rule approximation to integral.
	  g0=0.0d0
	    do 20 k=1,nint
	    g0=g0+f(a+(k+k-1)*h)
20	  continue
	  g0=0.5d0*g(1)+h*g0
	  h=0.5d0*h
	  nint=nint+nint
	  jmax=min(i,MAXJ)
	  fourj=1.0d0
	    do 30 j=1,jmax
c  Use Richardson extrapolation.
	    fourj=4.0d0*fourj
	    g1=g0+(g0-g(j))/(fourj-1.0d0)
	    g(j)=g0
	    g0=g1
30	  continue
	  if (abs(g0).gt.tol) then
	    error=(g0-gmax)/g0
	  else
	    error=(g0-gmax)/tol
	  end if
	  gmax=g0
	  g(jmax+1)=g0
	go to 10
40	rombint=g0
	if (i.gt.MAXITER.and.abs(error).gt.tol)
     2    write(*,*) 'Rombint failed to converge; integral, error=',
     3    rombint,error
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

	subroutine splder(y,dy,n)
c  Splder fits a cubic spline to y and returns the first derivatives at
c  the grid points in dy.  Dy is equivalent to a 4th-order Pade
c  difference formula for dy/di.
c
	implicit double precision (a-h,o-z)
	dimension f(3001),g(3001),y(n),dy(n)
	common /spline/ g
c
	n1=n-1
	if (n1.gt.3000) write(*,*) 'Spline array overflow!!! n1=',
     1n1,'>3000'
c  Quartic fit to dy/di at boundaries, assuming d3y/di3=0.
	f(1)=(-10.0d0*y(1)+15.0d0*y(2)-6.0d0*y(3)+y(4))/6.0d0
	f(n)=(10.0d0*y(n)-15.0d0*y(n1)+6.0d0*y(n-2)-y(n-3))/6.0d0
c  Solve the tridiagonal system
c  dy(i-1)+4*dy(i)+dy(i+1)=3*(y(i+1)-y(i-1)), i=2,3,...,n1,
c  with dy(1)=f(1), dy(n)=f(n).
	  do 10 i=2,n1
	  f(i)=g(i)*(3.0d0*(y(i+1)-y(i-1))-f(i-1))
10	continue
	dy(n)=f(n)
	  do 20 i=n1,1,-1
	  dy(i)=f(i)-g(i)*dy(i+1)
20	continue
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	subroutine splini
c  Splini must be called before splder to initialize array g in common.
c
	implicit double precision (a-h,o-z)
	dimension g(3001)
	common /spline/ g
	save /spline/
c
	g(1)=0.0d0
	  do 10 i=2,3001
	  g(i)=1.0d0/(4.0d0-g(i-1))
10	continue
	return
	end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c 
	function dozero(re,rexp,csbexp)
c
	implicit double precision (a-h,o-z)
	dimension value(100)
	fluxhalf=(re*re+csbexp*rexp*rexp)/2.d0
	if(re.gt.rexp*1.677d0.and.re.lt.rexp*1.678d0)then
	   dozero=re
	   return
	endif
	if(re.gt.rexp*1.679d0)then
	   start0=rexp*1.679
	   end0=re
	endif
	if(start0.lt.rexp*1.677d0)then
	   start0=re
	   end0=rexp*1.677d0
	endif
	checkstart=re*re*fluxb0(start0/re)
     1             +rexp*rexp*csbexp*fluxd0(start0/rexp)
     2             -fluxhalf
	if(checkstart.gt.0)then
	   write(*,*)'start>0!'
	   dozero=0.d0
	   return
	endif
	checkend=re*re*fluxb0(end0/re)
     1             +rexp*rexp*csbexp*fluxd0(end0/rexp)
     2             -fluxhalf
	if(checkend.lt.0)then
	   write(*,*)'end<0!'
	   dozero=0.d0
	   return
	endif
c	write(*,*)'check start,end',checkstart,checkend
	step=(end0-start0)/100.d0
	pos=start0
	do 1 i=1,101
	 pos=pos+step
	 value(i)=re*re*fluxb0(pos/re)
     1         +rexp*rexp*csbexp*fluxd0(pos/rexp)
     2             -fluxhalf   
	 if(value(i).gt.0.d0)then
	    dozero=pos-step*value(i)/(value(i)-value(i-1))
	    return
	 endif
 1	continue
	write(*,*)'Value always negative!'
	dozero=0.
	return
	end
C        PROGRAM MINCOG
C
C       ==========================================================
C       Purpose: This program computes the incomplete gamma
C                function r(a,x), â(a,x) and P(a,x) using
C                subroutine INCOG
C       Input :  a   --- Parameter
C                x   --- Argument
C       Output:  GIN --- r(a,x)
C                GIM --- â(a,x)
C                GIP --- P(a,x)
C       Example:
C            a     x      r(a,x)         â(a,x)         P(a,x)
C           -------------------------------------------------------
C           3.0   5.0  .17506960D+01  .24930404D+00  .87534798D+00
C       ===========================================================
C

        SUBROUTINE INCOG(A,X,GIN,GIM,GIP)
C
C       ===================================================
C       Purpose: Compute the incomplete gamma function
C                r(a,x), â(a,x) and P(a,x)
C       Input :  a   --- Parameter ( a ó 170 )
C                x   --- Argument 
C       Output:  GIN --- r(a,x)
C                GIM --- â(a,x)
C                GIP --- P(a,x)
C       Routine called: GAMMA for computing â(x)
C       ===================================================
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        XAM=-X+A*DLOG(X)
        IF (XAM.GT.700.0.OR.A.GT.170.0) THEN
           WRITE(*,*)'a and/or x too large'
           STOP
        ENDIF
        IF (X.EQ.0.0) THEN
           GIN=0.0
           CALL GAMMA(A,GA)
           GIM=GA
           GIP=0.0
        ELSE IF (X.LE.1.0+A) THEN
           S=1.0D0/A
           R=S
           DO 10 K=1,60
              R=R*X/(A+K)
              S=S+R
              IF (DABS(R/S).LT.1.0D-15) GO TO 15
10         CONTINUE
15         GIN=DEXP(XAM)*S
           CALL GAMMA(A,GA)
           GIP=GIN/GA
           GIM=GA-GIN
        ELSE IF (X.GT.1.0+A) THEN
           T0=0.0D0
           DO 20 K=60,1,-1
              T0=(K-A)/(1.0D0+K/(X+T0))
20         CONTINUE
           GIM=DEXP(XAM)/(X+T0)
           CALL GAMMA(A,GA)
           GIN=GA-GIM
           GIP=1.0D0-GIM/GA
        ENDIF
        END


        SUBROUTINE GAMMA(X,GA)
C
C       ==================================================
C       Purpose: Compute gamma function â(x)
C       Input :  x  --- Argument of â(x)
C                       ( x is not equal to 0,-1,-2,úúú)
C       Output:  GA --- â(x)
C       ==================================================
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION G(26)
        PI=3.141592653589793D0
        IF (X.EQ.INT(X)) THEN
           IF (X.GT.0.0D0) THEN
              GA=1.0D0
              M1=X-1
              DO 10 K=2,M1
10               GA=GA*K
           ELSE
              GA=1.0D+300
           ENDIF
        ELSE
           IF (DABS(X).GT.1.0D0) THEN
              Z=DABS(X)
              M=INT(Z)
              R=1.0D0
              DO 15 K=1,M
15               R=R*(Z-K)
              Z=Z-M
           ELSE
              Z=X
           ENDIF
           DATA G/1.0D0,0.5772156649015329D0,
     &          -0.6558780715202538D0, -0.420026350340952D-1,
     &          0.1665386113822915D0,-.421977345555443D-1,
     &          -.96219715278770D-2, .72189432466630D-2,
     &          -.11651675918591D-2, -.2152416741149D-3,
     &          .1280502823882D-3, -.201348547807D-4,
     &          -.12504934821D-5, .11330272320D-5,
     &          -.2056338417D-6, .61160950D-8,
     &          .50020075D-8, -.11812746D-8,
     &          .1043427D-9, .77823D-11,
     &          -.36968D-11, .51D-12,
     &          -.206D-13, -.54D-14, .14D-14, .1D-15/
           GR=G(26)
           DO 20 K=25,1,-1
20            GR=GR*Z+G(K)
           GA=1.0D0/(GR*Z)
           IF (DABS(X).GT.1.0D0) THEN
              GA=GA*R
              IF (X.LT.0.0D0) GA=-PI/(X*GA*DSIN(PI*X))
           ENDIF
        ENDIF
        RETURN
        END
