ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Computes Curves of Growth for Sersic profiles with given
c flattenings and determines the circularized half-luminosity ratio
c Author: R.P. Saglia, saglia@mpe.mpg.de, December 2010
c See Appendix of  Saglia et al. 2010, A&A, 524, A6    
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      Program CogModSersic
c
c Input parameters:
c aeb: bulge major axis effective scale lenght
c ahd: disk major axis exponential scale lenght
c bab: ellipticity b/a of the bulge
c bad: ellipticity b/a of the disk
c bt:  bulge to total ratio
c  
      implicit double precision (a-h,o-z)
      parameter (pi=3.141592654d0,twopi=2.0d0*pi)
      parameter (npoints=1000)
      external rombint,fb0
      real*8 aSer,aeb,babt
      real*8 vae(100000),vbab(100000),vaSer(100000)
      integer*4 ind(100000)
      character*80 namlist
c
c Output parameters
c re:  circularized half-luminosity radius
c
      real*8 re
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c fld: cog of disk
c
      real*8 rad(npoints),flb(npoints),flb0(npoints)
c
      common/cogs/rad,flb
      common/ellb/rb,bab
      common/nser/alpha,rkappa
c
      iman=0
      iman=0
      write(*,*)'******************************************************'
      write(*,*)'*          PROGRAM CogModSersic                      *'
      write(*,*)'******************************************************'
      write(*,*)'*       Author: R.P. Saglia, Deceomber 2010          *'
      write(*,*)'------------------------------------------------------'
      write(*,*)'See Appendix of  Saglia et al. 2010, A&A, 524, A6     '
      write(*,*)'Computes circularly averaged half-luminosity radii Re ' 
      write(*,*)'of a Sersic model with given flattening.              '
      write(*,*)'The input parameters are:                             '
      write(*,*)'nSer: the Sersic index (an interger or real number)   '
      write(*,*)'      nSer=4 corresponds to the R^1/4 law,            '
      write(*,*)'      nSer=1 corresponds to the exponential law       '
      write(*,*)'      nSer=0.5 corresponds to the Gaussian profile    '
      write(*,*)'aSer: the major axis half-luminosity radius           '
      write(*,*)'baSer: the minor to major axis ratio                  '
      write(*,*)'                                                      '
      write(*,*)'Give the parameters in the following order:           '
      write(*,*)'aSer,aeb,bab                                          '
      write(*,*)'                                                      '
      write(*,*)'If the list option is chosen, no starting comments are'
      write(*,*)'allowed.                                              '
      write(*,*)'                                                      '
      write(*,*)'The output is:                                        '
      write(*,*)'aeSer  nSer  (b/a)Ser ReCirSer Index                  '
      write(*,*)'where ReCirSer is the circularized radius and Index=0 '
      write(*,*)'flags failed circularitions and/or bad input values.  '
      write(*,*)'                                                      '
      write(*,*)'The output is written in the file                     '
      write(*,*)'                                                      '
      write(*,*)'manual.ReCirSerAll                                    '
      write(*,*)'                                                      '
      write(*,*)'if the manual entry is chosen, or                     '
      write(*,*)'                                                      '
      write(*,*)'filename.ReCirSerAll                                  '
      write(*,*)'                                                      '
      write(*,*)'if a list with name filename has been given.          '
      write(*,*)'******************************************************' 
      write(*,*)'Manual entry (0) or list (1)?'
      read(*,*),iman
      if(iman.eq.0)then
      write(*,*)'Input nSer,aeSer,baSer'
      read(*,*)aSer,aeb,bab
      nre=1
      vae(1)=aeb
      vbab(1)=bab
      vaSer(1)=aSer
      it=1
      if(vaSer(it).gt.0.and.vae(it).gt.0.and.vbab(it).ge.0
     1   .and.vbab(it).le.1)then
         ind(it)=1
      else
         ind(1)=0
      endif
c
c Compute cogs for Sersic profile 
c 
      else
         write(*,*)'List name?'
         read(*,'(a)')namlist
        call findspace(namlist,nspace)
         open(unit=10,file=namlist,status='old')
         it=1
         do i=1,100000
            read(10,*,end=11)vaSer(it),vae(it),vbab(it)
           if(vaSer(it).gt.0.and.vae(it).gt.0.and.vbab(it).ge.0
     1        .and.vbab(it).le.1)then
               ind(it)=1
            else
               ind(it)=0
            endif
            it=it+1
         enddo
 11      close(unit=10)
         nre=it-1
         if(nre.gt.100000)then
            write(*,*)nre,' graeter than 100000, set to 100000'
            nre=100000
         else
            write(*,*)nre,' galaxies to compute'
         endif
         step=0.01
         do i=1,npoints
            rad(i)=step*i
         enddo
      endif
c
c Compute re 
c
      step=0.01
      do i=1,npoints
         rad(i)=step*i
      enddo
      if(iman.eq.1)then
       open(unit=1,file=namlist(1:nspace)//'.ReCirSerAll',
     1      status='unknown')
      else
       open(unit=1,file='manual.ReCirSerAll',status='unknown')
      endif
      write(1,*)'    aeSer  nSer   (b/a)Ser  ReCirSer Index'
      ifirst=1
      isecond=1
      ithird=1
      write(*,*)'Starting to compute Re'
      do j=1,nre
         frac=1.d0*j/nre
c         write(*,*)'i=',j,' out of ',nre,' fraction=',frac
         if(frac.gt..25.and.frac.lt..5.and.ifirst.eq.1)then
            write(*,*)'Done 25%'
            ifirst=0
         endif
         if(frac.gt..50.and.frac.lt..75.and.isecond.eq.1)then
            write(*,*)'Done 50%'
            isecond=0
         endif
         if(frac.gt..75.and.frac.lt.1.and.ithird.eq.1)then
            write(*,*)'Done 75%'
            ithird=0
         endif
         aeb=vae(j)
         bab=vbab(j)
         aSer=vaSer(j)
         alpha=aSer
         rkappa=1.9992d0*alpha-0.3271d0
         if(ind(j).eq.1)then
         do i=1,npoints
            flb0(i)=fluxb0(rad(i))
            flb(i)=fluxb(rad(i))/twopi
c            write(*,*)i,rad(i),flb(i),flb0(i)
         enddo
c
         reSer=aeb*findhalf(flb)
c
         else
            reSer=-99.9
         endif
      write(*,*)'******************************************************'
      write(*,*)'* Sersic MJ ae=',aeb,' nSer=',aSer,' b/a=',bab,' Re=',
     1reSer
      write(*,*)'******************************************************'
         write(1,101)aeb,aSer,bab,reser,ind(j)
 101     format(f9.4,f7.4,f9.4,f9.4,i6)
      if(iman.eq.0)then
       open(unit=11,file='sersic.cogmod',status='unknown')
       write(11,*)'Sersic ae=',aeb,'nSer=',aSer,' b/a=',bab,' Re=',reSer
        do i=1,npoints
         write(11,*)aeb*rad(i),flb(i),flb0(i)
        enddo
       close(unit=11)
      endif
      enddo
      close(unit=1)
      stop
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c Finds first space character in a string
c
      subroutine findspace(nam,nnam)
      character*80 nam
      do i=1,80
         if(nam(i:i).eq.' ')goto 1
      enddo
 1    nnam=i-1
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function findhalf(fl)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=1000)
      real*8 rad(npoints),flb(npoints),fl(npoints)
c      
      common/cogs/rad,flb
c
      if(fl(npoints).lt..5)then
         findhalf=-1.d0
         goto 1
      endif
      do i=1,npoints
        if(fl(i).gt.0.5)then
            findhalf=rad(i-1)+(rad(i)-rad(i-1))
     1               *(0.5-fl(i-1))/(fl(i)-fl(i-1))
            goto 1
         endif
      enddo
c
 1    return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fluxb(x)
c
      implicit double precision (a-h,o-z)
      parameter (npoints=1000)
      real*8 rb,bab
c
c vectors containing:
c rad: radii
c flb: cog of bulge
c
      real*8 rad(npoints),flb(npoints)
      real*8 pi,tol
c
      external rombint,fb0
c
      common/cogs/rad,flb
      common/ellb/rb,bab
      common/nser/alpha,rkappa
c
c
      rb=x
      pi=3.141592654d0
      tol=1.d-5
      fluxb=4.d0*rombint(fb0,0.d0,pi/2.d0,tol)*bab
c
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      function fb0(phi)
c
      implicit double precision (a-h,o-z)
      real*8 rb,rr,bab,phi,geo
c
      common/ellb/rb,bab
      common/nser/alpha,rkappa
c
      geo=1.-(1.-bab**2)*cos(phi)**2
      rr=rb/bab*geo**.5
      fb0=fluxb0(rr)/geo
c
      return
      end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        function fluxb0(r)
c  Compute the integrated flux distribution for Sersic profile.
c  r is in units of half-luminosity radius.  Total flux is normalized to 1.
c
        implicit double precision (a-h,o-z)
c
        common/nser/alpha,rkappa
c
        X=rkappa*r**(1.d0/alpha)
        A=2.d0*alpha
        CALL INCOG(A,X,GIN,GIM,GIP)
        fluxb0=GIP
        return
        end
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

	function rombint(f,a,b,tol)
c  Rombint returns the integral from a to b of f(x)dx using Romberg integration.
c  The method converges provided that f(x) is continuous in (a,b).  The function
c  f must be double precision and must be declared external in the calling
c  routine.  tol indicates the desired relative accuracy in the integral.
c
	parameter (MAXITER=17,MAXJ=5)
	implicit double precision (a-h,o-z)
	dimension g(MAXJ+1)
	external f
c
	h=0.5d0*(b-a)
	gmax=h*(f(a)+f(b))
	g(1)=gmax
	nint=1
	error=1.0d20
	i=0
10	  i=i+1
	  if (i.gt.MAXITER.or.(i.gt.5.and.abs(error).lt.tol))
     2      go to 40
c  Calculate next trapezoidal rule approximation to integral.
	  g0=0.0d0
	    do 20 k=1,nint
	    g0=g0+f(a+(k+k-1)*h)
20	  continue
	  g0=0.5d0*g(1)+h*g0
	  h=0.5d0*h
	  nint=nint+nint
	  jmax=min(i,MAXJ)
	  fourj=1.0d0
	    do 30 j=1,jmax
c  Use Richardson extrapolation.
	    fourj=4.0d0*fourj
	    g1=g0+(g0-g(j))/(fourj-1.0d0)
	    g(j)=g0
	    g0=g1
30	  continue
	  if (abs(g0).gt.tol) then
	    error=(g0-gmax)/g0
	  else
	    error=(g0-gmax)/tol
	  end if
	  gmax=g0
	  g(jmax+1)=g0
	go to 10
40	rombint=g0
	if (i.gt.MAXITER.and.abs(error).gt.tol)
     2    write(*,*) 'Rombint failed to converge; integral, error=',
     3    rombint,error
	return
	end
C        PROGRAM MINCOG
C
C       ==========================================================
C       Purpose: This program computes the incomplete gamma
C                function r(a,x), â(a,x) and P(a,x) using
C                subroutine INCOG
C       Input :  a   --- Parameter
C                x   --- Argument
C       Output:  GIN --- r(a,x)
C                GIM --- â(a,x)
C                GIP --- P(a,x)
C       Example:
C            a     x      r(a,x)         â(a,x)         P(a,x)
C           -------------------------------------------------------
C           3.0   5.0  .17506960D+01  .24930404D+00  .87534798D+00
C       ===========================================================
C

        SUBROUTINE INCOG(A,X,GIN,GIM,GIP)
C
C       ===================================================
C       Purpose: Compute the incomplete gamma function
C                r(a,x), â(a,x) and P(a,x)
C       Input :  a   --- Parameter ( a ó 170 )
C                x   --- Argument 
C       Output:  GIN --- r(a,x)
C                GIM --- â(a,x)
C                GIP --- P(a,x)
C       Routine called: GAMMA for computing â(x)
C       ===================================================
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        XAM=-X+A*DLOG(X)
        IF (XAM.GT.700.0.OR.A.GT.170.0) THEN
           WRITE(*,*)'a and/or x too large'
           STOP
        ENDIF
        IF (X.EQ.0.0) THEN
           GIN=0.0
           CALL GAMMA(A,GA)
           GIM=GA
           GIP=0.0
        ELSE IF (X.LE.1.0+A) THEN
           S=1.0D0/A
           R=S
           DO 10 K=1,60
              R=R*X/(A+K)
              S=S+R
              IF (DABS(R/S).LT.1.0D-15) GO TO 15
10         CONTINUE
15         GIN=DEXP(XAM)*S
           CALL GAMMA(A,GA)
           GIP=GIN/GA
           GIM=GA-GIN
        ELSE IF (X.GT.1.0+A) THEN
           T0=0.0D0
           DO 20 K=60,1,-1
              T0=(K-A)/(1.0D0+K/(X+T0))
20         CONTINUE
           GIM=DEXP(XAM)/(X+T0)
           CALL GAMMA(A,GA)
           GIN=GA-GIM
           GIP=1.0D0-GIM/GA
        ENDIF
        END


        SUBROUTINE GAMMA(X,GA)
C
C       ==================================================
C       Purpose: Compute gamma function â(x)
C       Input :  x  --- Argument of â(x)
C                       ( x is not equal to 0,-1,-2,úúú)
C       Output:  GA --- â(x)
C       ==================================================
C
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        DIMENSION G(26)
        PI=3.141592653589793D0
        IF (X.EQ.INT(X)) THEN
           IF (X.GT.0.0D0) THEN
              GA=1.0D0
              M1=X-1
              DO 10 K=2,M1
10               GA=GA*K
           ELSE
              GA=1.0D+300
           ENDIF
        ELSE
           IF (DABS(X).GT.1.0D0) THEN
              Z=DABS(X)
              M=INT(Z)
              R=1.0D0
              DO 15 K=1,M
15               R=R*(Z-K)
              Z=Z-M
           ELSE
              Z=X
           ENDIF
           DATA G/1.0D0,0.5772156649015329D0,
     &          -0.6558780715202538D0, -0.420026350340952D-1,
     &          0.1665386113822915D0,-.421977345555443D-1,
     &          -.96219715278770D-2, .72189432466630D-2,
     &          -.11651675918591D-2, -.2152416741149D-3,
     &          .1280502823882D-3, -.201348547807D-4,
     &          -.12504934821D-5, .11330272320D-5,
     &          -.2056338417D-6, .61160950D-8,
     &          .50020075D-8, -.11812746D-8,
     &          .1043427D-9, .77823D-11,
     &          -.36968D-11, .51D-12,
     &          -.206D-13, -.54D-14, .14D-14, .1D-15/
           GR=G(26)
           DO 20 K=25,1,-1
20            GR=GR*Z+G(K)
           GA=1.0D0/(GR*Z)
           IF (DABS(X).GT.1.0D0) THEN
              GA=GA*R
              IF (X.LT.0.0D0) GA=-PI/(X*GA*DSIN(PI*X))
           ENDIF
        ENDIF
        RETURN
        END
